# SPDX-FileCopyrightText: 2024-present liuyanzhi08 <702368372@qq.com>
#
# SPDX-License-Identifier: MIT
# __version__ = "0.0.31" # 增加zmq支持
# __version__ = "0.0.32" # 抽取subscribe和publisher的原型
# __version__ = "0.0.33" # cal_exp增加safe_keys参数，传入以跳过'open'等关键词
# __version__ = "0.0.33" # fix zmq 的subscribe和publisher的context冲突
# __version__ = "0.0.34" # zmq支持是否server配置，允许1对多
# __version__ = "0.0.35" # get_trading_time默认返回原来交易所时区
# __version__ = "0.0.36" # get_trading_time默认返回原来交易所时区
# __version__ = "0.0.37" # 重构和修正cal_exp
# __version__ = "0.0.38" # cal_exp增加isin支持
# __version__ = "0.0.39" # cal_exp支持True+True -> 2
# __version__ = "0.0.41" # update version
# __version__ = "0.0.42" #修正cal_exp基本类型
# __version__ = "0.0.43" #cal_exp增加nanmin
__version__ = "0.0.44" # logger初始化时候就建立，而非第一次log才建立